#!/bin/bash

[ -z ${MAX_CONCURRENT} ] && MAX_CONCURRENT=1000
PMEM=0
[-z ${MAXTHREADS} ] && MAXTHREADS=0
[-z ${MAX_FDS} ] && MAX_FDS=0
_IFS="${IFS}"
IFS=$'\t'
s=0
SPACE="$[ s += 30 ],$[ s += 15 ],$[ s += 20 ],$[ s += 25 ],$[ s += 25 ]"
declare -A _SYSCTLV
declare -A TCP_MEM_BYTES
declare -A TCP_RMEM
declare -A TCP_WMEM
declare -A E_PORT

__err_report() {
    echo "Error on line $1"
}

# trap 'err_report $LINENO' ERR
#trap 'err_report ${BASH_SOURCE}:${FUNCNAME}:${LINENO}' ERR

trap 'echo "error: ${FUNCNAME}:${LINENO}"' ERR


_SYSCTLV[net.core.netdev_max_backlog]=""
_SYSCTLV[net.core.rmem_default]=""
_SYSCTLV[net.core.rmem_max]=""
_SYSCTLV[net.core.somaxconn]=""
_SYSCTLV[net.core.wmem_default]=""
_SYSCTLV[net.core.wmem_max]=""
_SYSCTLV[net.ipv4.ip_local_port_range]=""
_SYSCTLV[net.ipv4.tcp_abort_on_overflow]=""
_SYSCTLV[net.ipv4.tcp_adv_win_scale]=""
_SYSCTLV[net.ipv4.tcp_fin_timeout]=""
_SYSCTLV[net.ipv4.tcp_keepalive_intvl]=""
_SYSCTLV[net.ipv4.tcp_keepalive_probes]=""
_SYSCTLV[net.ipv4.tcp_keepalive_time]=""
_SYSCTLV[net.ipv4.tcp_max_syn_backlog]=""
_SYSCTLV[net.ipv4.tcp_mem]=""
_SYSCTLV[net.ipv4.tcp_mtu_probing]=""
_SYSCTLV[net.ipv4.tcp_rfc1337]=""
_SYSCTLV[net.ipv4.tcp_rmem]=""
_SYSCTLV[net.ipv4.tcp_tw_recycle]=""
_SYSCTLV[net.ipv4.tcp_tw_reuse]=""
_SYSCTLV[net.ipv4.tcp_window_scaling]=""
_SYSCTLV[net.ipv4.tcp_wmem]=""
_SYSCTLV[net.ipv4.tcp_max_syn_backlog]=""
_SYSCTLV[vm.swappiness]=""
_SYSCTLV[vm.overcommit_memory]=""
_SYSCTLV[vm.overcommit_ratio]=""


for K in "${!_SYSCTLV[@]}"; do _SYSCTLV[$K]=`sysctl -n $K`; done
#for K in "${!_SYSCTLV[@]}"; do echo $K=${_SYSCTLV[$K]}; done | sort -n

PROJECT_LOAD()
{
	MAXTHREADS=$[ 2 * MAX_CONCURRENT ]
	MAX_FDS=$[ 2 * MAXTHREADS ]
	echo "Projected Load:"
	echo MAXTHREADS: ${MAXTHREADS}
	echo MAX_FDS: ${MAX_FDS}
}

PHYSICAL_MEMORY()
{
# get physical memory available in the system
	PAGESIZE=`getconf PAGESIZE`
	PMEM=$((`getconf _PHYS_PAGES` * PAGESIZE ))
	echo SYSTEM_MEMORY: ${PMEM}
}

OPTIMIZE_SOCKET_BUFFERS()
{

	TCP_MEM_BYTES[high]=$[ PMEM * 25 / 100 ]
	TCP_MEM_BYTES[pressure]=$[ TCP_MEM_BYTES[high] * 66 / 100 ]
	TCP_MEM_BYTES[low]=$[ TCP_MEM_BYTES[high] * 50 / 100 ]
	
	echo TCP_MEM_BYTES: ${TCP_MEM_BYTES[*]}
	
	TCP_MEM[0]=$[ TCP_MEM_BYTES[low] / PAGESIZE ]
	TCP_MEM[1]=$[ TCP_MEM_BYTES[pressure] / PAGESIZE ]
	TCP_MEM[2]=$[ TCP_MEM_BYTES[high] / PAGESIZE ]

	_SYSCTLV[net.ipv4.tcp_mem]="${TCP_MEM[0]} ${TCP_MEM[1]} ${TCP_MEM[2]}"
	_SYSCTLV[net.core.rmem_default]=$[ TCP_MEM_BYTES[pressure] / MAX_FDS ]
	_SYSCTLV[net.core.wmem_default]=$[ TCP_MEM_BYTES[pressure] / MAX_FDS ]

	_SYSCTLV[net.core.rmem_max]=$[ 10 * TCP_MEM_BYTES[high] / MAX_FDS  ]
	_SYSCTLV[net.core.wmem_max]=$[ 10 * TCP_MEM_BYTES[high] / MAX_FDS  ]

	_SYSCTLV[net.ipv4.tcp_rmem]="${PAGESIZE} ${_SYSCTLV[net.core.rmem_default]} ${_SYSCTLV[net.core.rmem_max]}"
	_SYSCTLV[net.ipv4.tcp_wmem]="${PAGESIZE} ${_SYSCTLV[net.core.wmem_default]} ${_SYSCTLV[net.core.wmem_max]}"

	RECEIVE_SOCKET_BUFFERS="${_SYSCTLV[net.core.rmem_default]}"
	SEND_SOCKET_BUFFERS="${_SYSCTLV[net.core.wmem_default]}"
	
	_SYSCTLV[net.core.somaxconn]=$[ MAX_CONCURRENT * 1 ]
	_SYSCTLV[net.core.netdev_max_backlog]=$[ MAX_CONCURRENT * 200 ]
	_SYSCTLV[net.ipv4.tcp_max_syn_backlog]=$[ MAX_CONCURRENT * 1 ]
	_SYSCTLV[net.ipv4.tcp_mtu_probing]=1
	_SYSCTLV[net.ipv4.ip_local_port_range]="10000 65000"
	_SYSCTLV[net.ipv4.tcp_fin_timeout]=15
	_SYSCTLV[net.ipv4.tcp_rfc1337]=1
	_SYSCTLV[net.ipv4.tcp_abort_on_overflow]=1
	_SYSCTLV[net.ipv4.tcp_adv_win_scale]=1
	_SYSCTLV[net.ipv4.tcp_keepalive_intvl]=75
	_SYSCTLV[net.ipv4.tcp_keepalive_probes]=9
	_SYSCTLV[net.ipv4.tcp_keepalive_time]=900
	_SYSCTLV[net.ipv4.tcp_tw_recycle]=1
	_SYSCTLV[net.ipv4.tcp_tw_reuse]=1
	_SYSCTLV[net.ipv4.tcp_window_scaling]=1
	_SYSCTLV[vm.swappiness]=1
	_SYSCTLV[vm.overcommit_memory]=2
	_SYSCTLV[vm.overcommit_ratio]=60
		
	for K in "${!_SYSCTLV[@]}"
	do 
#		echo $K=${_SYSCTLV[$K]}; 
		sysctl -w $K=${_SYSCTLV[$K]}
	done 
	
#	for K in "${!_SYSCTLV[@]}"; do sysctl -w $K=${_SYSCTLV[$K]}; done 

return;
}

MANGO()
{

cat <<- _EOF 
RECEIVE_SOCKET_BUFFERS="${RECEIVE_SOCKET_BUFFERS}"
SEND_SOCKET_BUFFERS="${RECEIVE_SOCKET_BUFFERS}"
_EOF

}


MAIN()
{
	PHYSICAL_MEMORY
	PROJECT_LOAD	
	OPTIMIZE_SOCKET_BUFFERS
	[ "x${SYSTEM_INI}" != "x" ] && MANGO > ${SYSTEM_INI}
	
}

MAIN
